@@ -81,6 +81,9 @@ class SFA(_PanelToPanelTransformer):
         remove_repeat_words: boolean, default = False
             whether to use numerosity reduction (default False)
 
+        levels:              int, default = 1
+            Number of spatial pyramid levels
+
         save_words:          boolean, default = False
             whether to save the words generated for each series (default False)
 
@@ -113,6 +116,7 @@ class SFA(_PanelToPanelTransformer):
         levels=1,
         lower_bounding=True,
         save_words=False,
+        save_binning_dft=False,
         return_pandas_data_series=False,
     ):
         self.words = []
@@ -141,6 +145,8 @@ class SFA(_PanelToPanelTransformer):
 
         # TDE
         self.levels = levels
+        self.save_binning_dft = save_binning_dft
+        self.binning_dft = None
 
         #
         self.binning_method = binning_method
@@ -149,9 +155,9 @@ class SFA(_PanelToPanelTransformer):
         self.bigrams = bigrams
         self.skip_grams = skip_grams
 
-        # weighting for levels going up to 7 levels
-        # No real reason to go past 3
-        self.level_weights = [1, 2, 4, 16, 32, 64, 128]
+        # weighting for levels going up to 10 levels
+        # No real reason to go past 3, should probably not weight with too many.
+        self.level_weights = [1, 2, 4, 8, 16, 32, 64, 128, 256, 512]
 
         self.n_instances = 0
         self.series_length = 0
@@ -197,7 +203,7 @@ class SFA(_PanelToPanelTransformer):
         self._is_fitted = True
         return self
 
-    def transform(self, X, y=None):
+    def transform(self, X, y=None, supplied_dft=None):
         self.check_is_fitted()
         X = check_X(X, enforce_univariate=True, coerce_to_numpy=True)
         X = X.squeeze(1)
@@ -212,7 +218,10 @@ class SFA(_PanelToPanelTransformer):
 
         for i in range(X.shape[0]):
             # reuse 'transformed' array
-            dfts = self._mft(X[i, :], transformed, stds)
+            if supplied_dft is None:
+                dfts = self._mft(X[i, :], transformed, stds)
+            else:
+                dfts = supplied_dft[i]
 
             bag = defaultdict(int)
             # bag = Dict.empty(key_type=typeof((100,100.0)),
@@ -287,6 +296,8 @@ class SFA(_PanelToPanelTransformer):
                 for i in range(self.n_instances)
             ]
         )
+        if self.save_binning_dft:
+            self.binning_dft = dft
         dft = dft.reshape(len(X) * num_windows_per_inst, self.dft_length)
 
         if y is not None:
@@ -601,7 +612,7 @@ class SFA(_PanelToPanelTransformer):
             pos = window_ind + int((self.window_size / 2))
             quadrant = start + int(pos / quadrant_size)
 
-            bag[(word, quadrant)] += self.level_weights[i]
+            bag[word << 4 | quadrant] += self.level_weights[i]
 
             start += num_quadrants
 
